/*
  Copyright (C) 2000 Amanda J. Stent, stent@cs.rochester.edu

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.
*/

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

public class ArgumentationTool extends JFrame implements ListSelectionListener, ActionListener {
    private JTextArea comments;
    private DialogModel treeModel;
    private DynamicTree tree;
    JMenuBar menuBar;
    JMenu file, intentional, informational, adjacency, schemas, content;
    Vector con, contentRelations;
    JPanel pane;
    JFileChooser fc;

    public ArgumentationTool() {

	// Do frame stuff
	super("ArgumentationTool");

	pane = new JPanel();
	JScrollPane scrollPane;
	Container contentPane = getContentPane();
	JPanel tempPane;

	// if we exit
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
		System.exit(0);
            }
        });

	// setting up a pane for the top menu
	pane.setLayout(new BorderLayout());
	pane.setPreferredSize(new Dimension(600, 500));

	// setting up the top menu
	makeMenuBar();
	setJMenuBar(menuBar);
	//pane.add(menuBar, BorderLayout.NORTH);

	// making a place for the file to go
	tree = new DynamicTree();

	// making a text area to put in the tabbedpane
	tempPane = new JPanel();
	JToolBar toolBar = new JToolBar();
	makeTextToolBar(toolBar);
	tempPane.add(toolBar);
	tempPane.add(tree);

	pane.add(tempPane, BorderLayout.CENTER);

	comments = new JTextArea(3, 50);
	scrollPane = new JScrollPane(comments);
        tempPane = new JPanel();
	tempPane.add(scrollPane);

	pane.add(tempPane, BorderLayout.SOUTH);

	contentRelations = readList(new File("contentrelations.txt"));

	makeDialog(pane);
	contentPane.add(pane);
	//setContentPane(pane);
	fc = new JFileChooser();
	fc.addChoosableFileFilter(new TextFilter());
    }

    protected void makeDialog(Component comp) {
	
        con = readList(new File("content.txt"));
	ListDialog.initialize(comp, con, "Content chooser", "Content types");
	
    }

    protected JMenu makeMenu(String name, Vector items) {
	JMenu menu = new JMenu(name);
	JMenuItem menuItem;
	
	for(int i = 0; i < items.size(); i++) {
	    menuItem = new JMenuItem((String)items.get(i));
	    menuItem.addActionListener(this);
	    menu.add(menuItem);
	}
	
	return menu;
    }

    protected void makeMenuBar() {
	menuBar = new JMenuBar();
	setJMenuBar(menuBar);

	Vector temp = new Vector();
	temp.add("New");
	temp.add("Load");
	temp.add("Save");
	temp.add("Exit");
	temp.add("Info");
	file = makeMenu("File", temp);
	menuBar.add(file);

	temp = readList(new File("present.txt"));
	intentional = makeMenu("Presentational", temp);
	menuBar.add(intentional);

	temp = readList(new File("subject.txt"));
	informational = makeMenu("Subject-matter", temp);
	menuBar.add(informational);
	
	temp = readList(new File("adjacent.txt"));
	adjacency = makeMenu("Adjacency-pairs", temp);
	menuBar.add(adjacency);

	temp = readList(new File("schema.txt"));
	schemas = makeMenu("Schemas", temp);
	menuBar.add(schemas);

	temp = readList(new File("content.txt"));
	adjacency = makeMenu("Content-types", temp);
	menuBar.add(adjacency);
    }

    // In my window manager put a border around transient windows like dialog boxes -- AS
    public void actionPerformed(ActionEvent e) {
         JMenuItem source = (JMenuItem)(e.getSource());
	 String name = source.getText();
	 if(name.equals("New")) {
	     int returnVal = fc.showOpenDialog(ArgumentationTool.this);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
		    File filename = fc.getSelectedFile();
		    tree.clear();
		    tree.readDialog(filename);
                } else {
                }
	 } else if (name.equals("Load")) {
	     int returnVal = fc.showOpenDialog(ArgumentationTool.this);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
		    File filename = fc.getSelectedFile();
		    tree.clear();
		    tree.readTree(filename);
		    try {
			File comm = new File(filename.getPath().concat(".comments"));
			comm.createNewFile();
			comments.read(new FileReader(comm), null);
		    } catch (FileNotFoundException f) {
			System.err.println("Couldn't find file " + filename.getPath().concat(".comments"));
		    } catch (IOException f) { }
                } else {
                }
	 } else if (name.equals("Save")) {
	     int returnVal = fc.showSaveDialog(ArgumentationTool.this);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
		    File filename = fc.getSelectedFile();
		    tree.writeTree(filename);
		    try {
			File comm = new File(filename.getPath().concat(".comments"));
			comm.createNewFile();
			comments.write(new FileWriter(comm));
		    } catch (FileNotFoundException f) {
			System.err.println("Couldn't find file " + filename.getPath().concat(".comments"));
		    } catch (IOException f) { }
                } else {
                }
	 } else if (name.equals("Exit")) {
	     System.exit(0);
	 } else if (name.equals("Info")) {
	     JOptionPane.showMessageDialog(pane, "ArgumentationTool Copyright (C) 2000 Amanda Stent \nArgumentationTool comes with ABSOLUTELY NO WARRANTY. \nThis is free software, and you may redistribute it subject \nto the terms of the Free Software Foundation's General Public License. \nContact:  stent@cs.rochester.edu");
	 } else {
	     if(contentRelations.contains(name)) {
		 tree.addRelation(name, true);
	     } else {
		 tree.addRelation(name, false);
	     }
	 }
    }

    protected void makeTextToolBar(JToolBar toolBar) {
	JButton button = null;
	
	button = new JButton("Resegment");
	button.setToolTipText("Select a span, then duplicate it for resegmenting by clicking here.");
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    tree.duplicateCurrentNode();
		}
	    });
	toolBar.add(button);

	button = new JButton("Make span");
	button.setToolTipText("Select one or more span(s), then make a new span by clicking here.");
	button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		tree.addNode();
	    }
	});
	toolBar.add(button);
	
	button = new JButton("Delete span");
	button.setToolTipText("Select a span, then delete it by clicking here.");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tree.removeCurrentNode();
            }
        });
	toolBar.add(button);

	button = new JButton("Move span");
	button.setToolTipText("Select a span to move, select its destination, then move it by clicking here.");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tree.moveCurrentNode();
            }
        });
	toolBar.add(button);
	
	button = new JButton("Nucleify");
	button.setToolTipText("Declare selected span to be a nucleus.");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tree.nucleify();
            }
        });
	toolBar.add(button);

	button = new JButton("Discount");
	button.setToolTipText("Declare selected utterance to be discounted.");
        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tree.discount();
            }
        });
	toolBar.add(button);
    }

    public void valueChanged(ListSelectionEvent e) {
        if (e.getValueIsAdjusting())
            return;
        JList theList = (JList)e.getSource();
        if (theList.isSelectionEmpty()) {
	    displayResult("No value to select!");
        } else {
	    displayResult("You have selected " + theList.getSelectedValue());
        }
    }
		
    protected static Vector readList(File filename) {
	Vector listModel = new Vector();
	String temp;
	try {
	    BufferedReader reader = new BufferedReader(new FileReader(filename));
	    while ((temp = reader.readLine()) != null) {
		listModel.addElement(temp);
	    }
	    reader.close();
	} catch (FileNotFoundException e) { 
	    System.err.println("File not found:  " + filename);
	} catch (IOException e) {
	}
	return listModel;
    }

    protected void displayResult(String actionDescription) {
        comments.append(actionDescription + "\n");
    }

    public static void main(String[] args) {
        try {
	    // make it look java-ey
            UIManager.setLookAndFeel(
                UIManager.getCrossPlatformLookAndFeelClassName());
        } catch (Exception e) { }

        //Create the top-level container and add contents to it.
	ArgumentationTool frame = new ArgumentationTool();

        frame.pack();
        frame.setVisible(true);
    }
}


