/*
 * This code is based on the GenealogyTree code at the Sun Java tutorial site.

  Copyright (C) 2000 Amanda J. Stent, stent@cs.rochester.edu

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.
 */

import javax.swing.event.*;
import javax.swing.tree.*;
import java.util.Vector;

public class DialogModel implements TreeModel {
    private boolean showAncestors;
    private Vector treeModelListeners = new Vector();
    private Span rootSpan;

    public DialogModel(Span root) {
        rootSpan = root;
    }

    public void reload() { 
	fireTreeStructureChanged(rootSpan);
    }

    public void reload(Span s) { fireTreeNodesChanged(s); }

//////////////// Fire events //////////////////////////////////////////////

    protected void fireTreeStructureChanged(Span oldRoot) {
        int len = treeModelListeners.size();
        TreeModelEvent e = new TreeModelEvent(this, 
                                              new Object[] {oldRoot});
        for (int i = 0; i < len; i++) {
            ((TreeModelListener)treeModelListeners.elementAt(i)).
                    treeStructureChanged(e);
        }
    }

    protected void fireTreeNodesChanged(Span s) {
	int len = treeModelListeners.size();
	TreeModelEvent e = new TreeModelEvent(this, s.getPath());
	
	for (int i = 0; i < len; i++) {
	    ((TreeModelListener)treeModelListeners.elementAt(i)).treeNodesChanged(e);
	}
    }

//////////////// TreeModel interface implementation ///////////////////////


    /**
     * Adds a listener for the TreeModelEvent posted after the tree changes.
     */
    public void addTreeModelListener(TreeModelListener l) {
        treeModelListeners.addElement(l);
    }

    /**
     * Returns the child of parent at index index in the parent's child array.
     */
    public Object getChild(Object parent, int index) {
        Span p = (Span)parent;
        return p.getChildAt(index);
    }

    /**
     * Returns the number of children of parent.
     */
    public int getChildCount(Object parent) {
        Span p = (Span)parent;
        return p.getChildCount();
    }

    /**
     * Returns the index of child in parent.
     */
    public int getIndexOfChild(Object parent, Object child) {
        Span p = (Span)parent;
        return p.getIndexOfChild((Span)child);
    }

    /**
     * Returns the root of the tree.
     */
    public Object getRoot() {
        return rootSpan;
    }

    /**
     * Returns true if node is a leaf.
     */
    public boolean isLeaf(Object node) {
        Span p = (Span)node;
        return p.getChildCount() == 0;
    }

    /**
     * Removes a listener previously added with addTreeModelListener().
     */
    public void removeTreeModelListener(TreeModelListener l) {
        treeModelListeners.removeElement(l);
    }

    /**
     * Messaged when the user has altered the value for the item
     * identified by path to newValue.  Not used by this model.
     */
    public void valueForPathChanged(TreePath path, Object newValue) {
	Span currentNode = (Span)
	    (path.getLastPathComponent());
	String temp = (String)newValue;
	if (temp.startsWith("utt")) {
	    temp = temp.substring(temp.indexOf(':')+1);
	    temp = temp.trim();
	}
	currentNode.setLabel(temp);
        System.out.println("*** valueForPathChanged : "
                           + path + " --> " + newValue);
	reload(currentNode);
    }

    public void insertNodeInto(Object newChild, Object parent, int index) {
	((Span)parent).insertChildAt((Span)newChild, index);
	((Span)newChild).setParent((Span)parent);
    }

    public void removeNodeFromParent(Object child) {
	Span ch = (Span)child;
	Span parent = ch.getParent();
	parent.removeChild(ch);
	ch.setParent(null);
	if(parent.getChildCount() > 0) {
	    parent.setNumber(parent.getChildAt(0).getNumber());
	}
    }

}






